#version 330

//SETS THE RENDER MODE
// 0 = standard
// 1 = shadows
// 2 = particles
// 3+= nope! nothing there yet 
uniform int mode;

//Light
uniform struct Light
{	
	float intensity;
	float angle;
	vec3 direction;
	vec3 position;
	vec3 colour;
	mat4 pvMatrix;
	
} light;
uniform float transparency;
uniform sampler2DShadow shadowSampler;
uniform sampler2D textureSampler;
uniform vec3 specColor;
uniform float specularity;

in vec2 UV;
in vec4 shadowCoord;
in vec3 worldPosition;
in vec3 worldNormal;
in vec3 cameraEye;
in vec3 cameraNormal;
in vec3 cameraLightDir;
in vec3 relativeLightPosition;

out vec4 fragColor;
void main()
{
	if(mode != 1){
		
		vec3 texColor = texture(textureSampler, UV).rgb;
		vec3 diff = vec3(0);
		vec3 spec = vec3(0);
		vec3 amb = texColor*0.2;
	
		
		//float shadowCoeff = textureProj(shadowSampler, vec4(shadowCoord.xy, shadowCoord.z - bias, shadowCoord.w));
		
			
			float theta = 0;
			float specTheta = 0;
			float distance = length(cameraLightDir);
			float divisorForSpec = (distance*distance)/(light.intensity);
			float divisorForDiff = max(1, (distance*distance)/light.intensity);


			float effectiveLightAngle = acos(dot(normalize(-light.direction), normalize(relativeLightPosition)));

			//CHECKS IF FRAGMENT IS INSIDE CONE
			if(effectiveLightAngle < light.angle){


				vec3 n = normalize(cameraNormal);
				vec3 l = normalize(cameraLightDir);

				theta = clamp(dot(n, l), 0, 1);

				fragColor = vec4(0.5f,0.5f,0.5f,1);

				float bias = (0.00001/(0.2*log(shadowCoord.w)));
				//bias = bias * tan(acos(theta));
				float shadowCoeff = textureProj(shadowSampler, vec4(shadowCoord.xy, shadowCoord.z - bias, shadowCoord.w));

				//CHECKS IF FRAGMENT IS LIGHTED
				if(theta > 0.0){
					vec3 v = normalize(cameraEye);
					vec3 h = normalize(l + v);

					float specThetaTemp = clamp(dot(h, n), 0, 1);

					specTheta = pow(specThetaTemp, 32);
					specTheta = specTheta/divisorForSpec;
					theta = min(theta/divisorForDiff, 1);
				}

				spec = specColor*specTheta*specularity * shadowCoeff;
				diff = texColor*theta * shadowCoeff;	

				float coneMargin = light.angle - effectiveLightAngle ;
				if(coneMargin < effectiveLightAngle / 10){
					spec *= (coneMargin / (effectiveLightAngle / 10));
					diff *= (coneMargin / (effectiveLightAngle / 10));
				}
			}
	

		if(mode == 0) fragColor = vec4(amb+diff+spec, transparency);
	
		//fragColor = vec4(specColor, 0);
		//fragColor = vec4(distance/light.intensity, light.intensity/distance, 0, 0);
		//fragColor = vec4(dot(normalize(light.direction), normalize(-relativeLightPosition)), light.angle, 0, 1);
		if(mode == 2) fragColor = vec4(shadowCoord.z, shadowCoord.z, shadowCoord.z, 1);
		if(mode == 3) fragColor = vec4(transparency, 0, 0, 1);

	} else if (mode == 1) {
		fragColor = vec4(gl_FragCoord.z, gl_FragCoord.z, gl_FragCoord.z, 1);
	}
}