#version 330



//MVP
uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

//SETS THE RENDER MODE
// 0 = standard
// 1 = shadows
// 2+= nope! nothing there yet 
uniform int mode;

//Light
uniform struct Light
{
	float intensity;
	float angle;
	vec3 direction;
	vec3 position;
	vec3 colour;
	mat4 pvMatrix;
} light;

//Mesh
in vec2 UVcoord;
in vec3 position;
in vec3 normal;

//Out
out vec2 UV;
out vec4 shadowCoord;
out vec3 worldPosition;
out vec3 worldNormal;
out vec3 cameraEye;
out vec3 cameraNormal;
out vec3 cameraLightDir;
out vec3 relativeLightPosition;


void main()
{
	if(mode != 1){
		mat4 bias = mat4(0.5, 0.0, 0.0, 0.0,
						 0.0, 0.5, 0.0, 0.0,
						 0.0, 0.0, 0.5, 0.0,
						 0.5, 0.5, 0.5, 1.0);

		//CONSTANT
		UV = UVcoord;
		vec4 pos4 = vec4(position,1);
		vec4 shadowCoordt = (bias * light.pvMatrix * modelMatrix * pos4);
		shadowCoord = shadowCoordt;

		//WORLD SPACE
		pos4 = modelMatrix * pos4;

		worldPosition = pos4.xyz / pos4.w;
		worldNormal = (modelMatrix * vec4(normal, 0)).xyz;
		relativeLightPosition = light.position - worldPosition;

		//CAMERA SPACE
		pos4 = viewMatrix * pos4;
		cameraEye = vec3(0,0,0) - pos4.xyz;
		cameraNormal = (viewMatrix * modelMatrix * vec4(normal, 0)).xyz;
		cameraLightDir = (viewMatrix * vec4(light.position, 1.0)).xyz + cameraEye;

		//SCREEN SPACE
		gl_Position = projectionMatrix * pos4;
	
	} else if (mode == 1) {
		gl_Position = light.pvMatrix * modelMatrix * vec4(position, 1);
	}

}